/*=========================================================================

MEDITATIONS FOR THE CORONAPOCALYPSE

Daily digital meditations on a meditative image, pixel by pixel
Created during the period of "social distancing" for the
Covid19 Corona Virus pandemic - a way to recenter in eccentric times
©2020, Eric Valosin

*FOR AN ENHANCED, IMMERSIVE EXPERIENCE* 
view in a pitch dark room with white walls
on a large monitor, TV, or digital projector.
Observe the walls and ceiling (perhaps even facing away from screen)


**FEEL FREE TO USE, SHARE, OR ALTER, IN PART OR IN WHOLE, 
BUT PLEASE INCLUDE THE FOLLOWING ATTRIBUTION IN YOUR CODE:
    Based upon "Meditations for the Coronapocalypse" by Eric Valosin, 2020. 
    www.ericvalosin.com • Used with permission of the artist

*****************************************

UPDATES in this version (1.0.1)

• Fixed Gray Screen Glitch in Exported App  
  • removed problematic saveString() and loadString() functions, along with the .txt file it created and referenced
    to increment the daily image, which were causing problems upon exporting.
  • Replaced it with code that identifies the the current day in absolute terms and where it fits in
    a 51 day cycle since the beginning of the Common Era (accounting for leap years!) to advance the image each day.

*****************************************

============================================================================*/

PImage pic;

// Variables for colors
int i;
int r;
int g;
int b;
int lastR;
int lastG;
int lastB;

// Variable for crossfade steps
int fadeinStep; // timer for crossfade
int fadeOut; // for tinting original image to black
float RchangePerStep;
float GchangePerStep;
float BchangePerStep;

void setup(){
  i = int(random(0, 9999)); // starting pixel, chosen at random. set for 100x100 image 
  r = 0;
  g = 0;
  b = 0;
  lastR = 0;
  lastG = 0;
  lastB = 0;
  
  fadeinStep = 0;
  fadeOut = 255;

////////////// Check what day it is and load corresponding image ///////////////////////
  int dayCounter; 
  int totalDaysSinceCE;
  int daysinCompletedYrs;
  int dayofYear;
  int monthVal = 0;
  int febVal;
    
  // find number of leap years to date since CE
  int leapYrs = 0;
  for (int yr = 1; yr < year(); yr++){
    if (yr % 4 == 0 && yr % 100 != 0){
      leapYrs++;
    } else if (yr % 4 == 0 && yr % 100 == 0 && yr % 400 == 0){
      leapYrs++;
    }
  }
  
  //add up total days in all completed years since CE
  int nonLeapYrs = (year()-1)-leapYrs;
  int LYdays = leapYrs*366;
  int nonLYdays = nonLeapYrs*365;
  daysinCompletedYrs = LYdays + nonLYdays;
  
  // For this year, check for leap year and assign appropriate number of days to February
  if (year() % 4 == 0 && year() % 100 != 0){
    febVal = 29;
  } else if (year() % 4 == 0 && year() % 100 == 0 && year() % 400 == 0){
    febVal = 29;
  } else {
    febVal = 28;
  }  
  
  // Find how many days have elapsed in completed months this year
  if (month() == 1){
    monthVal = 0;
  }
  if (month() == 2){
    monthVal = 31;
  }
  if (month() == 3){
    monthVal = febVal + 31;
  }
  if (month() == 4){
    monthVal = febVal + 62;
  }
  if (month() == 5){
    monthVal = febVal + 92;
  }
  if (month() == 6){
    monthVal = febVal + 123;
  }
  if (month() == 7){
    monthVal = febVal + 153;
  }
  if (month() == 8){
    monthVal = febVal + 184;
  }
  if (month() == 9){
    monthVal = febVal + 215;
  }
  if (month() == 10){
    monthVal = febVal + 245;
  }
  if (month() == 11){
    monthVal = febVal + 276;
  }
  if (month() == 12){
    monthVal = febVal + 306;
  }
  // add to that the number of completed days this month
  dayofYear = monthVal + day();
  totalDaysSinceCE = daysinCompletedYrs + dayofYear;
  
  // see where that falls in a cycle of 51 days
  dayCounter = totalDaysSinceCE % 51;
 
//=========================
// DEBUG DAY COUNTER
  //println(month()+"/"+day()+"/"+year()); 
  //println("Completed leap years to date: " + leapYrs + "; total days: " + daysinCompletedYrs);
  //println("Day of the current year: " + dayofYear);
  //println("Today is the " + totalDaysSinceCE + "th day of the Common Era");
  //println("location in 51 day cycle: " + dayCounter);
//==========================

  // load image that corresponds with the day
  pic=loadImage(dayCounter + ".jpg"); 
 
/////////////////////////////////////////////////

  // set up display
  size(displayWidth, displayHeight);
  background(0);
  imageMode(CENTER);
}
 
void draw(){
  // uses millis() to time the events 
  
  if(millis() >= 2000 && millis() <= 8000){
      image(pic, width/2, height/2, 600, 600);
  }
  
  // highlight starting pixel  
  if(millis() >= 4000 && millis() <= 12600){ 
    // get location of selected i pixel
    int y = int(i/100);
    int x = int(i - y * 100);
    // highlight pixel
    noFill();
    stroke(255,255,0);
    rect((((width/2)-300)+(x*6-1)), (((height/2)-300)+(y*6-1)), 8, 8); // forms bounding box around scaled pixel
    // ^generic formula = (x-1, y-1, 3, 3); x and y scaled in this case by 6 since the output is 6x the resolution of the reference image, size = 6 plus one pixel on each side 
   }
  
  //begin fade to black
  if(millis() >= 8000 && fadeOut != 0){
    tint(fadeOut,fadeOut,fadeOut);
    image(pic, width/2, height/2, 600,600);
    fadeOut--;
   
    // continue to show pixel as fade happens
    int y = int(i/100);
    int x = int(i - y * 100);
    noFill();
    stroke(255,255,0); 
    rect((((width/2)-300)+(x*6-1)), (((height/2)-300)+(y*6-1)), 8, 8);
  }
  
  //get colors from selected pixel
  pic.loadPixels();
  r = int(red(pic.pixels[i]));
  g = int(green(pic.pixels[i]));
  b = int(blue(pic.pixels[i]));
  
  // Establish rate of change on first step of crossfade
  if(fadeOut==0){
    if(fadeinStep == 0){
      RchangePerStep = (float(r) - float(lastR))/60; //change 60 to desired steps in crossfade
      GchangePerStep = (float(g) - float(lastG))/60;
      BchangePerStep = (float(b) - float(lastB))/60;
      
      if(RchangePerStep > 0){RchangePerStep = ceil(RchangePerStep);}
      if(RchangePerStep < 0){RchangePerStep = floor(RchangePerStep);}
      
      if(GchangePerStep > 0){GchangePerStep = ceil(GchangePerStep);}
      if(GchangePerStep < 0){GchangePerStep = floor(GchangePerStep);}
      
      if(BchangePerStep > 0){BchangePerStep = ceil(BchangePerStep);}
      if(BchangePerStep < 0){BchangePerStep = floor(BchangePerStep);}
    }
    
    // Adjust colors incrementally until it reaches rests on the next pixel's colors by the final step of the crossfade
    if(fadeinStep < 60){ //change 60 to desired steps 
      noStroke();
     
      //constrain values so they don't exceed target rgb values
      if(RchangePerStep > 0){lastR=constrain(lastR,0,r-int(RchangePerStep));} // not sure why it adds the change per step one too many times, but this fixes it
      if(RchangePerStep < 0){lastR=constrain(lastR,r-int(RchangePerStep),255);}
      
      if(GchangePerStep > 0){lastG=constrain(lastG,0,g-int(GchangePerStep));}
      if(GchangePerStep < 0){lastG=constrain(lastG,g-int(GchangePerStep),255);}
      
      if(BchangePerStep > 0){lastB=constrain(lastB,0,b-int(BchangePerStep));}
      if(BchangePerStep < 0){lastB=constrain(lastB,b-int(BchangePerStep),255);}
      
      fill(round(lastR), round(lastG), round(lastB));
      rect(0, 0, width, height);

      lastR += RchangePerStep;
      lastG += GchangePerStep;
      lastB += BchangePerStep;
        
      fadeinStep++;
    
      //On final step of crossfade make sure new pixel color has been reached and iterate to next pixel
      if(fadeinStep == 60){ //change 60 to desired steps
        lastR = r;
        lastG = g;
        lastB = b;
      
        fadeinStep = 0;
        delay(2000); // how long to sit on each pixel before crossfading
        
        i++;
        if(i > 9999){
          i = 0;
        }
      }    
      delay(4000/60); // "frame rate" of crossfade (duration of crossfade/steps)

//==========================================
// FOR DEBUGGING PIXEL COLOR TRANSITIONS:
// println("Step "+fadeinStep+", lastR "+lastR+", r "+r+", RchangePerStep "+RchangePerStep+"  |||  lastG "+lastG+", g "+g+", GchangePerStep "+GchangePerStep+"  |||  lastB "+lastB+", b "+b+", BchangePerStep "+BchangePerStep);
//==========================================
    } 
  } 
}
